@@ -219,6 +219,13 @@ class ApiAutomator:
 
         self._logger.debug("Waiting for User to be ready for inclusion in AssumeRolePolicy")
 
+        # attach the data access policy to the group
+        iam_client.attach_group_policy(
+            GroupName=group_name,
+            PolicyArn=policy_arn
+        )
+        self._logger.info(f"Attached Policy {policy_arn} to Group {group_name}")
+
         role_created = False
         retries = 0
         while role_created is False and retries < 5:
@@ -265,7 +272,7 @@ class ApiAutomator:
                     PolicyArn=policy_arn
                 )
                 policy_attached = True
-                self._logger.info(f"Attached Policy {policy_arn} to {role_name}")
+                self._logger.info(f"Attached Policy {policy_arn} to Role {role_name}")
             except iam_client.exceptions.MalformedPolicyDocumentException as mpde:
                 if "Invalid principal" in str(mpde):
                     # this is raised when something within IAM hasn't yet propagated correctly.
@@ -513,15 +520,52 @@ class ApiAutomator:
 
         return all_tables
 
-    def update_glue_catalog_resource_policy(self, region: str, producer_account_id: str, consumer_account_id: str,
-                                            database_name: str, tables: list):
+    def write_glue_catalog_resource_policy(self, policy: dict, current_hash: str = None):
+        '''
+        Write a new glue catalog policy document. This is a low level interface that just performs the mechanic of
+        correctly writing the supplied policy, including where a hash must be supplied.
+        :param policy:
+        :param current_hash:
+        :return:
+        '''
         glue_client = self._get_client('glue')
-        new_resource_policy = None
-        current_resource_policy = None
+
         try:
             current_resource_policy = glue_client.get_resource_policy()
+
+            # if no external hash has been provided, then just use the current hash from the doc.
+            if current_hash is None:
+                current_hash = current_resource_policy.get('PolicyHash')
+
+            glue_client.put_resource_policy(
+                PolicyInJson=json.dumps(policy),
+                PolicyHashCondition=current_hash,
+                PolicyExistsCondition='MUST_EXIST',
+                EnableHybrid='TRUE'
+            )
         except glue_client.exceptions.EntityNotFoundException:
-            pass
+            # write the resource policy as new
+            glue_client.put_resource_policy(
+                PolicyInJson=json.dumps(policy),
+                PolicyExistsCondition='NOT_EXIST',
+                EnableHybrid='TRUE'
+            )
+
+    def get_current_glue_policy(self):
+        glue_client = self._get_client('glue')
+
+        try:
+            current_resource_policy = glue_client.get_resource_policy()
+            glue_policy = json.loads(current_resource_policy.get('PolicyInJson'))
+            current_hash = current_resource_policy.get('PolicyHash')
+
+            return glue_policy, current_hash
+        except glue_client.exceptions.EntityNotFoundException:
+            return None, None
+
+    def add_tbac_glue_catalog_resource_policy(self, region: str, producer_account_id: str, consumer_account_id: str,
+                                              database_name: str, tables: list):
+        current_resource_policy, current_hash = self.get_current_glue_policy()
 
         cf = {
             'region': region,
@@ -535,44 +579,33 @@ class ApiAutomator:
         cf['table_list'] = tables
         policy = json.loads(utils.generate_policy('lf_cross_account_tbac.pystache', config=cf))
 
-        policy_condition = None
         if current_resource_policy is None:
             new_resource_policy = {
                 "Version": "2012-10-17",
                 "Statement": policy
             }
-            glue_client.put_resource_policy(
-                PolicyInJson=json.dumps(new_resource_policy),
-                PolicyExistsCondition='NOT_EXIST',
-                EnableHybrid='TRUE'
+            self.write_glue_catalog_resource_policy(
+                policy=new_resource_policy
             )
             self._logger.info(
                 f"Created new Catalog Resource Policy on {producer_account_id} allowing Tag Based Access by {consumer_account_id}")
         else:
-            new_resource_policy = json.loads(current_resource_policy.get('PolicyInJson'))
-            current_hash = current_resource_policy.get('PolicyHash')
-
             update_statement, policy_index, did_modification = self._get_glue_resource_policy_statement_to_modify(
                 region=region,
-                policy=new_resource_policy, producer_account_id=producer_account_id,
+                policy=current_resource_policy, producer_account_id=producer_account_id,
                 consumer_account_id=consumer_account_id,
                 database_name=database_name, tables=tables
             )
 
             # add the new statement
             if update_statement is None:
-                new_resource_policy['Statement'].append(policy)
+                current_resource_policy['Statement'].append(policy)
                 did_modification = True
             elif update_statement is not None:
-                new_resource_policy['Statement'][policy_index] = update_statement
+                current_resource_policy['Statement'][policy_index] = update_statement
 
             if did_modification is True:
-                glue_client.put_resource_policy(
-                    PolicyInJson=json.dumps(new_resource_policy),
-                    PolicyHashCondition=current_hash,
-                    PolicyExistsCondition='MUST_EXIST',
-                    EnableHybrid='TRUE'
-                )
+                self.write_glue_catalog_resource_policy(current_hash=current_hash, policy=current_resource_policy)
                 self._logger.info(
                     f"Updated Catalog Resource Policy on {producer_account_id} allowing Tag Based Access by {consumer_account_id}")
 
@@ -917,36 +950,36 @@ class ApiAutomator:
         try:
             glue_client.get_crawler(Name=crawler_name)
         except glue_client.exceptions.from_code('EntityNotFoundException'):
-            glue_client.create_crawler(
-                Name=crawler_name,
-                Role=crawler_role,
-                DatabaseName=database_name,
-                Description="S3 Crawler to sync structure of %s.%s to Data Mesh" % (database_name, table_name),
-                Targets={
-                    'S3Targets': [
-                        {
-                            'Path': s3_location
-                        },
-                    ]
-                },
-                Schedule="cron(0 */4 * * ? *)" if sync_schedule is None else sync_schedule,
-                SchemaChangePolicy={
-                    'UpdateBehavior': 'LOG',
-                    'DeleteBehavior': 'LOG'
-                },
-                RecrawlPolicy={
-                    'RecrawlBehavior': 'CRAWL_NEW_FOLDERS_ONLY'
-                },
-                LineageConfiguration={
-                    'CrawlerLineageSettings': 'ENABLE' if enable_lineage is True else 'DISABLE'
-                },
-                Tags=DEFAULT_TAGS
-            )
-            self._logger.info("Created new Glue Crawler %s" % crawler_name)
-
-        # create lakeformation permissions in the mesh account for the glue crawler role
-
-        # create s3 permission for glue crawler role
+            try:
+                glue_client.create_crawler(
+                    Name=crawler_name,
+                    Role=crawler_role,
+                    DatabaseName=database_name,
+                    Description="S3 Crawler to sync structure of %s.%s to Data Mesh" % (database_name, table_name),
+                    Targets={
+                        'S3Targets': [
+                            {
+                                'Path': s3_location
+                            },
+                        ]
+                    },
+                    Schedule="cron(0 */4 * * ? *)" if sync_schedule is None else sync_schedule,
+                    SchemaChangePolicy={
+                        'UpdateBehavior': 'LOG',
+                        'DeleteBehavior': 'LOG'
+                    },
+                    RecrawlPolicy={
+                        'RecrawlBehavior': 'CRAWL_NEW_FOLDERS_ONLY'
+                    },
+                    LineageConfiguration={
+                        'CrawlerLineageSettings': 'ENABLE' if enable_lineage is True else 'DISABLE'
+                    },
+                    Tags=DEFAULT_TAGS
+                )
+                self._logger.info("Created new Glue Crawler %s" % crawler_name)
+            except glue_client.exceptions.AccessDeniedException as ade:
+                self._logger.error(
+                    "Cannot create Glue Crawler - caller doesn't have permissions or is missing iam::PassRole")
 
         return crawler_name
 
@@ -992,8 +1025,11 @@ class ApiAutomator:
                 **args
             )
 
+            this_account_id = self._get_client('sts').get_caller_identity().get('Account')
+
             # tag the database with default tags
-            db_arn = utils.get_db_arn(region_name=glue_client.meta.region_name, database_name=database_name)
+            db_arn = utils.get_db_arn(region_name=glue_client.meta.region_name, catalog_id=this_account_id,
+                                      database_name=database_name)
             glue_client.tag_resource(
                 ResourceArn=db_arn,
                 TagsToAdd=DEFAULT_TAGS
@@ -1144,14 +1180,54 @@ class ApiAutomator:
         # put the policy back into the bucket store
         s3_client.put_bucket_policy(Bucket=bucket_name, Policy=json.dumps(new_policy))
 
+    def accept_lf_resource_shares(self, share_list: list) -> tuple:
+        '''
+        Causes a list of RAM shares to be accepted by the caller
+        :param share_list:
+        :return:
+        '''
+        ram_client = self._get_client('ram')
+
+        get_response = ram_client.get_resource_share_invitations()
+
+        # only accept peding lakeformation shares from the source account
+        shares_accepted = []
+        shares_active = []
+        shares_not_found = []
+        for r in get_response.get('resourceShareInvitations'):
+            share_arn = r.get('resourceShareArn')
+            if share_arn in share_list:
+                if r.get('status') == 'PENDING':
+                    ram_client.accept_resource_share_invitation(
+                        resourceShareInvitationArn=r.get('resourceShareInvitationArn')
+                    )
+                    shares_accepted.append(share_arn)
+                    self._logger.info(f"Accepted RAM Share {share_arn}")
+                elif r.get('status') == 'ACCEPTED':
+                    shares_active.append(share_arn)
+                else:
+                    shares_not_found.append(share_arn)
+            else:
+                shares_not_found.append(share_arn)
+
+        return shares_accepted, shares_active, shares_not_found
+
     def accept_pending_lf_resource_shares(self, sender_account: str, filter_resource_arn: str = None):
+        '''
+        Accepts all pending resource shares of the specified type from the sending account. Used to automatically accept
+        shares from the data mesh back to the producer
+
+        :param sender_account:
+        :param filter_resource_arn:
+        :return:
+        '''
         ram_client = self._get_client('ram')
 
         get_response = ram_client.get_resource_share_invitations()
 
         accepted_share = False
         for r in get_response.get('resourceShareInvitations'):
-            # only accept peding lakeformation shares from the source account
+            # only accept pending lakeformation shares from the source account
             if r.get('senderAccountId') == sender_account and 'LakeFormation' in r.get('resourceShareName') and r.get(
                     'status') == 'PENDING':
                 if filter_resource_arn is None or r.get('resourceShareArn') == filter_resource_arn:
